"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var TYPE = Symbol('anyOf');
/**
Array of type

Usage:

   {
     a: SchemaType.Any
   }

See schema.js for more information
*/

var _default = function _default() {
  return Object.freeze({
    $$type: TYPE,
    validatorFactory: validatorFactory.apply(void 0, arguments)
  });
};

exports["default"] = _default;

function validatorFactory() {
  for (var _len = arguments.length, allowedTypes = new Array(_len), _key = 0; _key < _len; _key++) {
    allowedTypes[_key] = arguments[_key];
  }

  if (!allowedTypes.filter(function (type) {
    return type;
  }).length) {
    throw new TypeError('Missing types');
  }
  /**
  Validation function favtory
   @param fieldName {String}           the field name being eva
  @param field {Object}               the field options
  @param schema {Schema}              the schema instance
  @param wrappedValidator {Function}  (optional) the validator being wrapped
  */


  return function anyOfValidator(fieldName, field, schema, wrappedValidator) {
    var _field$required = field.required,
        required = _field$required === void 0 ? false : _field$required,
        _field$nullable = field.nullable,
        nullable = _field$nullable === void 0 ? true : _field$nullable;
    var baseFields = allowedTypes.map(function (baseType) {
      return schema._normalizeField(baseType, fieldName);
    });
    var itemValidators = baseFields.map(function (baseField) {
      return baseField.type.validatorFactory(fieldName, field, schema);
    });
    return function validator(value, self, context) {
      if (value === undefined && required) {
        return 'required';
      } else if (value === null && !nullable) {
        return 'isNull';
      } else if (value !== undefined && value !== null) {
        var message;
        var validFound = false;
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = itemValidators[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var itemValidator = _step.value;

            var _message = itemValidator(value);

            if (!_message) {
              validFound = true;
            } else if (!message) {
              message = _message;
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator["return"] != null) {
              _iterator["return"]();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        if (!validFound && typeof message === 'string') {
          return message;
        }
      }

      return wrappedValidator && wrappedValidator(value, self, context);
    };
  };
}