"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

/**
Number type

Usage:

   {
     a: Number
   }

See schema.js for more information
*/
var _default = Object.freeze({
  $$type: Symbol('number'),
  $$primitive: Number,
  validatorFactory: numberValidator
});
/**
Validation function favtory

@param fieldName {String}           the field name being eva
@param field {Object}               the field options
@param schema {Schema}              the schema instance
@param wrappedValidator {Function}  (optional) the validator being wrapped
*/


exports["default"] = _default;

function numberValidator(fieldName, field, schema, wrappedValidator) {
  var _field$required = field.required,
      required = _field$required === void 0 ? false : _field$required,
      _field$nullable = field.nullable,
      nullable = _field$nullable === void 0 ? true : _field$nullable,
      _field$minNumber = field.minNumber,
      minNumber = _field$minNumber === void 0 ? -Infinity : _field$minNumber,
      _field$maxNumber = field.maxNumber,
      maxNumber = _field$maxNumber === void 0 ? Infinity : _field$maxNumber;
  return function validator(value, self, context) {
    if (value === undefined && required) {
      return 'required';
    } else if (value === null && !nullable) {
      return 'isNull';
    } else if (value !== undefined && value !== null) {
      if (typeof value !== 'number' || isNaN(value)) {
        return 'invalidType';
      } else if (value < minNumber) {
        return 'minNumber';
      } else if (value > maxNumber) {
        return 'maxNumber';
      }
    }

    return wrappedValidator && wrappedValidator(value, self, context);
  };
}