"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

/**
String type

Usage:

   {
     a: String
   }

See schema.js for more information
*/
var _default = Object.freeze({
  $$type: Symbol('string'),
  $$primitive: String,
  validatorFactory: stringValidator
});
/**
Validation function favtory

@param fieldName {String}           the field name being eva
@param field {Object}               the field options
@param schema {Schema}              the schema instance
@param wrappedValidator {Function}  (optional) the validator being wrapped
*/


exports["default"] = _default;

function stringValidator(fieldName, field, schema, wrappedValidator) {
  var _field$required = field.required,
      required = _field$required === void 0 ? false : _field$required,
      _field$nullable = field.nullable,
      nullable = _field$nullable === void 0 ? true : _field$nullable,
      _field$minLength = field.minLength,
      minLength = _field$minLength === void 0 ? 0 : _field$minLength,
      _field$maxLength = field.maxLength,
      maxLength = _field$maxLength === void 0 ? Infinity : _field$maxLength;
  return function validator(value, self, context) {
    if (value === undefined && required) {
      return 'required';
    } else if (value === null && !nullable) {
      return 'isNull';
    } else if (value !== undefined && value !== null) {
      if (typeof value !== 'string') {
        return 'invalidType';
      } else if (value.length < minLength) {
        return 'minLength';
      } else if (value.length > maxLength) {
        return 'maxLength';
      }
    }

    return wrappedValidator && wrappedValidator(value, self, context);
  };
}