"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _varValidator = _interopRequireDefault(require("var-validator"));

var _context = _interopRequireDefault(require("./context"));

var _model = require("./model");

var _types = require("./types");

// field validator config
_varValidator["default"].enableScope = false;
_varValidator["default"].enableBrackets = false; // schema id counter

var schemaCount = 0;

var Schema =
/*#__PURE__*/
function (_TypeProvider) {
  (0, _inherits2["default"])(Schema, _TypeProvider);
  (0, _createClass2["default"])(Schema, null, [{
    key: "use",

    /**
    Add a plugin to use with new instances of Schema. Added
    plugins do not affect currently instanciated instances.
     @param plugin {Function} a single function receiving the instance
    */
    value: function use(pluginFactory) {
      var plugin = pluginFactory(Schema);

      if (plugin) {
        if (typeof plugin === 'function') {
          Schema._plugins.push({
            init: plugin
          });
        } else {
          Schema._plugins.push(plugin);
        }
      }

      return Schema;
    }
    /**
    Create a new instance
     @param fields {Object} the fields definition (will be sanitized and normalized)
    @params options {Object} the schema options
    */

  }]);

  function Schema(fields) {
    var _this;

    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck2["default"])(this, Schema);
    _this = (0, _possibleConstructorReturn2["default"])(this, (0, _getPrototypeOf2["default"])(Schema).call(this));

    if (!fields || !Object.keys(fields).length) {
      throw new TypeError('No defined fields');
    } else if ((0, _typeof2["default"])(fields) !== 'object') {
      throw new TypeError('Invalid fields argument');
    }

    Schema._plugins.forEach(function (plugin) {
      return plugin.preInit && plugin.preInit((0, _assertThisInitialized2["default"])(_this), fields, options);
    });

    Object.defineProperties((0, _assertThisInitialized2["default"])(_this), {
      options: {
        enumerable: true,
        configurable: false,
        writable: false,
        value: options
      },
      fields: {
        enumerable: true,
        configurable: false,
        writable: false,
        value: normalizeValidators(fields, (0, _assertThisInitialized2["default"])(_this))
      },
      fieldNames: {
        enumerable: true,
        configurable: false,
        writable: false,
        value: Object.keys(fields)
      },
      _type: {
        enumerable: false,
        configurable: false,
        writable: false,
        value: createType((0, _assertThisInitialized2["default"])(_this))
      },
      _namedContexts: {
        enumerable: false,
        configurable: false,
        writable: false,
        value: {}
      }
    });

    Schema._plugins.forEach(function (plugin) {
      return plugin.init && plugin.init((0, _assertThisInitialized2["default"])(_this));
    });

    _this.fieldNames.forEach(function (field) {
      return Object.freeze(_this.fields[field]);
    });

    Object.freeze(_this.fields); // no further mods!

    Object.freeze(_this.fieldNames); //

    return _this;
  }
  /**
  Return the type for this schema
   @return {type}
  */


  (0, _createClass2["default"])(Schema, [{
    key: "getType",
    value: function getType() {
      return this._type;
    }
    /**
    Create a new empty model from the fields' default values specification
     @return {Object}
    */

  }, {
    key: "createModel",
    value: function createModel(data) {
      var _this2 = this;

      var model = (0, _model.createModel)(this, data);

      Schema._plugins.forEach(function (plugin) {
        return plugin.extendModel && plugin.extendModel(model, _this2);
      });

      return model;
    }
    /**
    Create a new validation context based on this schema
     @param name {String}   (optional) return a shared context identified by name
    */

  }, {
    key: "createContext",
    value: function createContext(name) {
      var _this3 = this;

      var newContext = !name || !this._namedContexts[name];
      var context = newContext ? new _context["default"](this) : this._namedContexts[name];

      if (newContext) {
        Schema._plugins.forEach(function (plugin) {
          return plugin.extendContext && plugin.extendContext(context, _this3);
        });

        if (name) {
          this._namedContexts[name] = context;
        }
      }

      return context;
    }
  }]);
  return Schema;
}(_types.TypeProvider); // internal properties


Object.defineProperties(Schema, {
  _plugins: {
    enumerable: false,
    configurable: false,
    writable: false,
    value: []
  },
  _normalizeField: {
    enumerable: false,
    configurable: false,
    writable: false,
    value: normalizeField
  }
});
Object.defineProperties(Schema.prototype, {
  _normalizeField: {
    enumerable: false,
    configurable: false,
    writable: false,
    value: normalizeField
  }
});
/**
Create a type for the given schema

@param schemaType {Schema}
@return {type}
*/

function createType(schemaType) {
  return {
    $$type: Symbol('schema' + ++schemaCount),
    validatorFactory: function validatorFactory(fieldName, field, schema, wrappedValidator) {
      var _field$required = field.required,
          required = _field$required === void 0 ? false : _field$required,
          _field$nullable = field.nullable,
          nullable = _field$nullable === void 0 ? true : _field$nullable;
      var validatorContext = schemaType.createContext();

      function validator(value, options, context) {
        if (value === undefined) {
          return required ? 'required' : undefined;
        } else if (value === null) {
          return !nullable ? 'isNull' : undefined;
        } else {
          try {
            if (!validatorContext.validate(value)) {
              if (fieldName) {
                var contextMessages = validatorContext.getMessages();
                Object.keys(contextMessages).forEach(function (subFieldName) {
                  return context.setMessage(fieldName + '.' + subFieldName, contextMessages[subFieldName]);
                });
              }

              return 'invalid';
            }
          } catch (e) {
            return 'invalidType';
          }
        }

        return wrappedValidator && wrappedValidator(value, options, context);
      }

      validator.context = validatorContext;
      return validator;
    }
  };
}
/**
Sanitize all fields from the given object, make sure that each
key is a valid name, and that each type if a recognized validator

@param fields {object}
@param schema {Schema}
@return {Object}
*/


function normalizeValidators(fields, schema) {
  var fieldNames = Object.keys(fields);

  for (var _i = 0, _fieldNames = fieldNames; _i < _fieldNames.length; _i++) {
    var fieldName = _fieldNames[_i];

    if (!_varValidator["default"].isValid(fieldName)) {
      throw new Error('Invalid field name : ' + fieldName);
    }

    var field = fields[fieldName] = normalizeField(fields[fieldName], fieldName);
    field.validator = field.type.validatorFactory(fieldName, field, schema, field.validator);
  }

  return fields;
}
/**
Return an object that is normalized with a valid type property

@param field
@param fieldName {String}   (optional) the field name
@return {Object}
*/


function normalizeField(field, fieldName) {
  if (!field) {
    throw new TypeError('Empty field specification' + (fieldName ? ' for ' + fieldName : ''));
  }

  if ((0, _types.isType)(field)) {
    field = {
      type: (0, _types.getType)(field)
    };
  } else if ((0, _types.isType)(field.type)) {
    field.type = (0, _types.getType)(field.type);
  } else if ((0, _typeof2["default"])(field) === 'object') {
    var typeSchema = new Schema(field);
    field = {
      type: typeSchema.getType(),
      schema: typeSchema
    };
  }

  if (!field.type || !field.type.$$type) {
    throw new TypeError('Invalid field specification' + (fieldName ? ' for ' + fieldName : ''));
  }

  return field;
}

var _default = Schema;
exports["default"] = _default;