"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _array = _interopRequireDefault(require("./array"));

var TYPE = Symbol('arrayOf');
/**
Array of type

Usage:

   {
     a: SchemaType.Any
   }

See schema.js for more information
*/

var _default = function _default(type) {
  return Object.freeze({
    $$type: TYPE,
    validatorFactory: validatorFactory(type)
  });
};

exports["default"] = _default;

function validatorFactory(baseType) {
  if (!baseType) {
    throw new TypeError('Missing type');
  }
  /**
  Validation function favtory
   @param fieldName {String}           the field name being eva
  @param field {Object}               the field options
  @param schema {Schema}              the schema instance
  @param wrappedValidator {Function}  (optional) the validator being wrapped
  */


  return function arrayOfValidator(fieldName, field, schema, wrappedValidator) {
    var _field$timeout = field.timeout,
        timeout = _field$timeout === void 0 ? 200 : _field$timeout;

    var baseField = schema._normalizeField(baseType, fieldName + '.*');

    var itemValidator = baseField.type.validatorFactory(null, baseField, schema);
    return _array["default"].validatorFactory(fieldName, field, schema, function (value, self, context) {
      if (value) {
        var expires = Date.now() + timeout;

        var _loop = function _loop(i, len) {
          var item = value[i];
          var itemContext = itemValidator.context;
          var message = itemValidator(item, self, itemContext);

          if (typeof message === 'string') {
            if (itemContext && fieldName) {
              var contextMessages = itemContext.getMessages();
              Object.keys(contextMessages).forEach(function (subFieldName) {
                return context.setMessage(fieldName + '.' + i + '.' + subFieldName, contextMessages[subFieldName]);
              });
            }

            return {
              v: 'invalid'
            };
          }
        };

        for (var i = 0, len = value.length; i < len && expires > Date.now(); ++i) {
          var _ret = _loop(i, len);

          if ((0, _typeof2["default"])(_ret) === "object") return _ret.v;
        }
      }

      return wrappedValidator && wrappedValidator(value, self, context);
    });
  };
}