"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

/**
Array type

Usage:

   {
     a: Array
   }

See schema.js for more information
*/
var _default = Object.freeze({
  $$type: Symbol('array'),
  $$primitive: Array,
  validatorFactory: arrayValidator
});
/**
Validation function favtory

@param fieldName {String}           the field name being eva
@param field {Object}               the field options
@param schema {Schema}              the schema instance
@param wrappedValidator {Function}  (optional) the validator being wrapped
*/


exports["default"] = _default;

function arrayValidator(fieldName, field, schema, wrappedValidator) {
  var _field$required = field.required,
      required = _field$required === void 0 ? false : _field$required,
      _field$nullable = field.nullable,
      nullable = _field$nullable === void 0 ? true : _field$nullable,
      _field$minCount = field.minCount,
      minCount = _field$minCount === void 0 ? 0 : _field$minCount,
      _field$maxCount = field.maxCount,
      maxCount = _field$maxCount === void 0 ? Infinity : _field$maxCount;
  return function validator(value, self, context) {
    if (value === undefined && required) {
      return 'required';
    } else if (value === null && !nullable) {
      return 'isNull';
    } else if (value !== undefined && value !== null) {
      if (!Array.isArray(value)) {
        return 'invalidType';
      } else if (value.length < minCount) {
        return 'minCount';
      } else if (value.length > maxCount) {
        return 'maxCount';
      }
    }

    return wrappedValidator && wrappedValidator(value, self, context);
  };
}