"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

/**
Date type

Usage:

   {
     a: Date
   }

See schema.js for more information
*/
var _default = Object.freeze({
  $$type: Symbol('date'),
  $$primitive: Date,
  validatorFactory: dateValidator
});
/**
Validation function favtory

@param fieldName {String}           the field name being eva
@param field {Object}               the field options
@param schema {Schema}              the schema instance
@param wrappedValidator {Function}  (optional) the validator being wrapped
*/


exports["default"] = _default;

function dateValidator(fieldName, field, schema, wrappedValidator) {
  var _field$required = field.required,
      required = _field$required === void 0 ? false : _field$required,
      _field$nullable = field.nullable,
      nullable = _field$nullable === void 0 ? true : _field$nullable,
      _field$minDate = field.minDate,
      minDate = _field$minDate === void 0 ? -Infinity : _field$minDate,
      _field$maxDate = field.maxDate,
      maxDate = _field$maxDate === void 0 ? Infinity : _field$maxDate;
  return function validator(value, self, context) {
    if (value === undefined && required) {
      return 'required';
    } else if (value === null && !nullable) {
      return 'isNull';
    } else if (value !== undefined && value !== null) {
      if (!(value instanceof Date)) {
        return 'invalidType';
      } else if (isNaN(value.getTime())) {
        return 'invalidDate';
      } else if (value < minDate) {
        return 'minDate';
      } else if (value > maxDate) {
        return 'maxDate';
      }
    }

    return wrappedValidator && wrappedValidator(value, self, context);
  };
}